==> shogun_0.3.2.orig.tar.gz: shogun_0.3.2.orig/src/classifier/svm/LibSVM.cpp <==
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Written (W) 1999-2007 Soeren Sonnenburg
 * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
 */

#include "classifier/svm/LibSVM.h"
#include "lib/io.h"

CLibSVM::CLibSVM() : CSVM()
{
	model=NULL;
}

CLibSVM::CLibSVM(DREAL C, CKernel* k, CLabels* lab)
{
	model=NULL;

	set_C(C,C);
	set_labels(lab);
	set_kernel(k);
}

CLibSVM::~CLibSVM()
{
	free(model);
}

bool CLibSVM::train()
{
	free(model);

	struct svm_node* x_space;

	ASSERT(get_labels() && get_labels()->get_num_labels());
	problem.l=get_labels()->get_num_labels();
	SG_INFO( "%d trainlabels\n", problem.l);

	problem.y=new double[problem.l];
	problem.x=new struct svm_node*[problem.l];
	x_space=new struct svm_node[2*problem.l];

	ASSERT(problem.y);
	ASSERT(problem.x);
	ASSERT(x_space);

	for (int i=0; i<problem.l; i++)
	{
		problem.y[i]=get_labels()->get_label(i);
		problem.x[i]=&x_space[2*i];
		x_space[2*i].index=i;
		x_space[2*i+1].index=-1;
	}

	int weights_label[2]={-1,+1};
	double weights[2]={1.0,get_C2()/get_C1()};

	ASSERT(get_kernel());

	param.svm_type=C_SVC; // C SVM
	param.kernel_type = LINEAR;
	param.degree = 3;
	param.gamma = 0;	// 1/k
	param.coef0 = 0;
	param.nu = 0.5;
	param.kernel=get_kernel();
	param.cache_size = get_kernel()->get_cache_size();
	param.C = get_C1();
	param.eps = epsilon;
	param.p = 0.1;
	param.shrinking = 1;
	param.nr_weight = 2;
	param.weight_label = weights_label;
	param.weight = weights;

	const char* error_msg = svm_check_parameter(&problem,&param);

	if(error_msg)
	{
		fprintf(stderr,"Error: %s\n",error_msg);
		exit(1);
	}

	model = svm_train(&problem, &param);

	if (model)
	{
		ASSERT(model->nr_class==2);
		ASSERT( (model->l==0) || (model->l > 0 && model->SV && model->sv_coef && model->sv_coef[0]) );

		int num_sv=model->l;

		create_new_model(num_sv);
		CSVM::set_objective(model->objective);

		//workaround for libsvm bug (?)
		//if the first example has a positive label the whole decision function
		//is inverted, i.e. f(x) becomes -f(x)

		if (problem.y[0]>=0)
		{
			SG_WARNING( "inverting libsvm's decision function as first label is >= 0\n");
			set_bias(-model->rho[0]);

			for (int i=0; i<num_sv; i++)
			{
				set_support_vector(i, (model->SV[i])->index);
				set_alpha(i, model->sv_coef[0][i]);
			}
		}
		else
		{
			set_bias(model->rho[0]);

			for (int i=0; i<num_sv; i++)
			{
				set_support_vector(i, (model->SV[i])->index);
				set_alpha(i, -model->sv_coef[0][i]);
			}
		}

		delete[] problem.x;
		delete[] problem.y;
		delete[] x_space;

		return true;
	}
	else
		return false;
}

==> shogun_0.4.4.orig.tar.gz: shogun_0.4.4.orig/src/classifier/svm/LibSVM.cpp <==
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Written (W) 1999-2007 Soeren Sonnenburg
 * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
 */

#include "classifier/svm/LibSVM.h"
#include "lib/io.h"

CLibSVM::CLibSVM() : CSVM(), model(NULL)
{
}

CLibSVM::CLibSVM(DREAL C, CKernel* k, CLabels* lab) : CSVM(C, k, lab), model(NULL)
{
}

CLibSVM::~CLibSVM()
{
	//SG_PRINT("deleting LibSVM\n");
}

bool CLibSVM::train()
{

	struct svm_node* x_space;

	ASSERT(get_labels() && get_labels()->get_num_labels());
	ASSERT(get_labels()->is_two_class_labeling());

	problem.l=get_labels()->get_num_labels();
	SG_INFO( "%d trainlabels\n", problem.l);

	problem.y=new double[problem.l];
	problem.x=new struct svm_node*[problem.l];
	x_space=new struct svm_node[2*problem.l];

	ASSERT(problem.y);
	ASSERT(problem.x);
	ASSERT(x_space);

	for (int i=0; i<problem.l; i++)
	{
		problem.y[i]=get_labels()->get_label(i);
		problem.x[i]=&x_space[2*i];
		x_space[2*i].index=i;
		x_space[2*i+1].index=-1;
	}

	int weights_label[2]={-1,+1};
	double weights[2]={1.0,get_C2()/get_C1()};

	ASSERT(get_kernel() && get_kernel()->get_lhs());
    ASSERT(get_kernel()->get_lhs()->get_num_vectors() == problem.l);

	param.svm_type=C_SVC; // C SVM
	param.kernel_type = LINEAR;
	param.degree = 3;
	param.gamma = 0;	// 1/k
	param.coef0 = 0;
	param.nu = 0.5;
	param.kernel=get_kernel();
	param.cache_size = get_kernel()->get_cache_size();
	param.C = get_C1();
	param.eps = epsilon;
	param.p = 0.1;
	param.shrinking = 1;
	param.nr_weight = 2;
	param.weight_label = weights_label;
	param.weight = weights;

	const char* error_msg = svm_check_parameter(&problem,&param);

	if(error_msg)
	{
		fprintf(stderr,"Error: %s\n",error_msg);
		exit(1);
	}

	model = svm_train(&problem, &param);

	if (model)
	{
		ASSERT(model->nr_class==2);
		ASSERT( (model->l==0) || (model->l > 0 && model->SV && model->sv_coef && model->sv_coef[0]) );

		int num_sv=model->l;

		create_new_model(num_sv);
		CSVM::set_objective(model->objective);

		DREAL sgn=model->label[0];

		set_bias(-sgn*model->rho[0]);

		for (int i=0; i<num_sv; i++)
		{
			set_support_vector(i, (model->SV[i])->index);
			set_alpha(i, sgn*model->sv_coef[0][i]);
		}

		delete[] problem.x;
		delete[] problem.y;
		delete[] x_space;

		free(model);
		model=NULL;
		return true;
	}
	else
		return false;
}
